#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import requests
from typing import List
import time
import webbrowser
import threading

import wx
import wx.adv
from loguru import logger

from ..aipy.trustoken import TrustTokenAPI
from .. import T

class InitialProviderPage(wx.adv.WizardPage):
    def __init__(self, parent, provider_config):
        super().__init__(parent)
        self.provider_config = provider_config
        self.init_ui()
        self.SetSize(800, 600)

    def init_ui(self):
        vbox = wx.BoxSizer(wx.VERTICAL)

        # 标题
        title = wx.StaticText(self, label=T('Select LLM Provider'))
        title.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        vbox.Add(title, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Provider 选择
        provider_box = wx.StaticBox(self, label=T('Provider'))
        provider_sizer = wx.StaticBoxSizer(provider_box, wx.VERTICAL)

        self.provider_choice = wx.Choice(
            self,
            choices=['Trustoken', T('Other')]
        )
        self.provider_choice.SetSelection(0)  # 默认选中 TrustToken
        self.provider_choice.Bind(wx.EVT_CHOICE, self.on_provider_selected)
        provider_sizer.Add(self.provider_choice, 0, wx.EXPAND | wx.ALL, 5)
        vbox.Add(provider_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # 提示信息
        self.hint = wx.StaticText(self, label="")
        # 设置文本自动换行
        self.hint.Wrap(400)
        vbox.Add(self.hint, 0, wx.ALL | wx.EXPAND, 10)

        self.SetSizer(vbox)
        # 设置初始提示信息
        self.on_provider_selected(None)

    def on_provider_selected(self, event):
        provider = self.provider_choice.GetStringSelection()
        if provider == 'Trustoken':
            self.hint.SetLabel(f"""{T('Trustoken is an intelligent API Key management service')}：

1. {T('Auto get and manage API Key')}
2. {T('Support multiple LLM providers')}
3. {T('Auto select optimal model')}
4. {T('Recommended for beginners, easy to configure and feature-rich')}""")
        else:
            self.hint.SetLabel(f"""{T('Select other providers requires manual configuration')}：

1. {T('Need to apply for API Key yourself')}
2. {T('Manually input API Key')}
3. {T('Manually select model')}""")
        # 重新计算换行
        self.hint.Wrap(400)

    def get_selection(self):
        # 获取当前选择，如果没有选择则返回默认值
        selection = self.provider_choice.GetStringSelection()
        if not selection:
            # 如果获取不到选择，使用 GetSelection() 获取索引
            index = self.provider_choice.GetSelection()
            if index >= 0:
                selection = self.provider_choice.GetString(index)
            else:
                selection = 'Trustoken'  # 如果索引也无效，返回默认值
        return selection

    def GetNext(self):
        selection = self.get_selection()
        if selection == "Trustoken":
            return self.GetParent().trust_token_page
        else:
            return self.GetParent().provider_page

    def GetPrev(self):
        return None

class TrustTokenPage(wx.adv.WizardPage):
    def __init__(self, parent, provider_config):
        super().__init__(parent)
        self.provider_config = provider_config
        self.api = TrustTokenAPI()
        self.poll_interval = 5
        self.request_id = None
        self.polling_thread = None
        self.stop_polling = False
        self.start_time = None
        self.polling_timeout = 310  # 5分钟10秒
        self.init_ui()
        self.SetSize(800, 600)
        # 绑定页面切换事件
        self.GetParent().Bind(wx.adv.EVT_WIZARD_PAGE_CHANGING, self.on_page_changing)

    def init_ui(self):
        vbox = wx.BoxSizer(wx.VERTICAL)

        # 标题
        title = wx.StaticText(self, label=T('Trustoken Configuration'))
        title.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        vbox.Add(title, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # API Key 显示
        api_key_box = wx.StaticBox(self, label="API Key")
        api_key_sizer = wx.StaticBoxSizer(api_key_box, wx.VERTICAL)

        self.api_key_text = wx.TextCtrl(
            self,
            style=wx.TE_READONLY
        )
        api_key_sizer.Add(self.api_key_text, 1, wx.EXPAND | wx.ALL, 5)
        vbox.Add(api_key_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # 获取按钮
        self.fetch_button = wx.Button(self, label=T('Auto open browser to get API Key'))
        self.fetch_button.Bind(wx.EVT_BUTTON, self.on_fetch)
        vbox.Add(self.fetch_button, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # 状态信息
        self.status_text = wx.StaticText(self, label="")
        vbox.Add(self.status_text, 0, wx.ALL | wx.EXPAND, 5)

        self.url_ctrl = wx.adv.HyperlinkCtrl(self, label=T('You can also click this link to open browser to get API Key'), style=wx.adv.HL_ALIGN_LEFT | wx.adv.HL_CONTEXTMENU)
        self.url_ctrl.Hide()
        vbox.Add(self.url_ctrl, 0, wx.ALL, 5)

        vbox.AddSpacer(20)

        # 剩余时间文本
        self.time_text = wx.StaticText(self, label='')
        vbox.Add(self.time_text, 0, wx.EXPAND | wx.ALL, 5)

        # 进度条
        self.progress_bar = wx.Gauge(self, range=100)
        vbox.Add(self.progress_bar, 0, wx.EXPAND | wx.ALL, 5)

        self._toggle_progress(False)
        self.SetSizer(vbox)

    def _toggle_progress(self, show: bool):
        self.progress_bar.Show(show)
        self.time_text.Show(show)

    def _update_progress(self):
        if not self.start_time:
            return

        elapsed = time.time() - self.start_time
        if elapsed >= self.polling_timeout:
            progress = 100
            time_remaining = 0
        else:
            progress = int((elapsed / self.polling_timeout) * 100)
            time_remaining = int(self.polling_timeout - elapsed)

        wx.CallAfter(self.progress_bar.SetValue, progress)
        wx.CallAfter(self.time_text.SetLabel, f"{T('Remaining time')}: {time_remaining}{T('seconds')}")

    def _poll_status(self, save_func):
        self.start_time = time.time()
        while not self.stop_polling and time.time() - self.start_time < self.polling_timeout:
            self._update_progress()

            data = self.api.check_status(self.request_id)
            if not data:
                time.sleep(self.poll_interval)
                continue

            status = data.get('status')
            wx.CallAfter(self._update_status, f"{T('Current status')}: {status}")

            if status == 'approved':
                if save_func:
                    save_func(data['secret_token'])
                wx.CallAfter(self._on_success)
                return True
            elif status == 'expired':
                wx.CallAfter(self._update_status, T('Binding expired'))
                wx.CallAfter(self._on_failure)
                return False
            elif status == 'pending':
                pass
            else:
                wx.CallAfter(self._update_status, f"{T('Unknown status')}: {status}")
                wx.CallAfter(self._on_failure)
                return False

            time.sleep(self.poll_interval)

        if not self.stop_polling:
            wx.CallAfter(self._update_status, T('Waiting timeout'))
            wx.CallAfter(self._on_failure)
        return False

    def _update_status(self, status):
        self.status_text.SetLabel(status)

    def _on_success(self):
        self._toggle_progress(False)
        self.fetch_button.Enable()
        self.status_text.SetLabel(T("API Key obtained successfully!"))
        self.Layout()  # 强制重新布局

    def _on_failure(self):
        self._toggle_progress(False)
        self.fetch_button.Enable()
        self.status_text.SetLabel(T('API Key acquisition failed, please try again'))
        self.Layout()  # 强制重新布局

    def on_fetch(self, event):
        self.status_text.SetLabel(T('Requesting binding'))
        self.fetch_button.Disable()

        data = self.api.request_binding()
        if not data:
            self._on_failure()
            return

        approval_url = data['approval_url']
        self.request_id = data['request_id']
        expires_in = data['expires_in']
        self.polling_timeout = expires_in
        self._update_status(T('Waiting for user confirmation'))

        # 打开浏览器
        self.url_ctrl.SetURL(approval_url)
        self.url_ctrl.Show()
        webbrowser.open(approval_url)

        # 显示进度组并重新布局
        self._toggle_progress(True)
        self.Layout()

        # 开始轮询
        self.polling_thread = threading.Thread(
            target=self._poll_status,
            args=(self._save_token,)
        )
        self.polling_thread.daemon = True
        self.polling_thread.start()

    def _save_token(self, token):
        self.api_key_text.SetValue(token)

    def get_api_key(self):
        return self.api_key_text.GetValue()

    def on_page_changing(self, event):
        # 如果是当前页面，且是向前切换
        if event.GetPage() == self and event.GetDirection():
            api_key = self.get_api_key()
            if not api_key:
                wx.MessageBox(T('Please get API Key first'), T('Hint'), wx.OK | wx.ICON_INFORMATION)
                event.Veto()
                return
        event.Skip()

    def GetNext(self):
        return self.GetParent().model_page

    def GetPrev(self):
        return self.GetParent().initial_page

class ProviderPage(wx.adv.WizardPage):
    def __init__(self, parent, provider_config):
        super().__init__(parent)
        self.provider_config = provider_config
        self.init_ui()
        self.SetSize(800, 600)
        self.GetParent().Bind(wx.adv.EVT_WIZARD_PAGE_CHANGING, self.on_page_changing)

    def init_ui(self):
        vbox = wx.BoxSizer(wx.VERTICAL)

        # 标题
        title = wx.StaticText(self, label=T('Select LLM Provider'))
        title.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        vbox.Add(title, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Provider 选择
        provider_box = wx.StaticBox(self, label=T('Provider'))
        provider_sizer = wx.StaticBoxSizer(provider_box, wx.VERTICAL)

        # 过滤掉 TrustToken
        providers = [name for name in self.provider_config.providers.keys() if name != "Trustoken"]
        self.provider_choice = wx.Choice(
            self,
            choices=providers
        )
        if providers:  # 确保列表不为空
            self.provider_choice.SetSelection(0)  # 默认选中第一个选项
        self.provider_choice.Bind(wx.EVT_CHOICE, self.on_provider_selected)
        provider_sizer.Add(self.provider_choice, 0, wx.EXPAND | wx.ALL, 5)
        vbox.Add(provider_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # API Key 输入
        api_key_box = wx.StaticBox(self, label="API Key")
        api_key_sizer = wx.StaticBoxSizer(api_key_box, wx.VERTICAL)

        self.api_key_text = wx.TextCtrl(
            self,
            size=(400, -1)
        )
        api_key_sizer.Add(self.api_key_text, 0, wx.EXPAND | wx.ALL, 5)
        vbox.Add(api_key_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # 提示信息
        hint = wx.StaticText(self, label=T('Please select provider and input API Key, click Next to verify'))
        hint.SetForegroundColour(wx.Colour(100, 100, 100))
        vbox.Add(hint, 0, wx.ALL, 10)

        self.SetSizer(vbox)

    def on_provider_selected(self, event):
        provider = self.provider_choice.GetStringSelection()
        if provider in self.provider_config.config:
            config = self.provider_config.config[provider]
            self.api_key_text.SetValue(config["api_key"])

    def get_provider(self):
        return self.provider_choice.GetStringSelection()

    def get_api_key(self):
        return self.api_key_text.GetValue()

    def GetNext(self):
        return self.GetParent().model_page

    def GetPrev(self):
        return self.GetParent().initial_page

    def on_page_changing(self, event):
        # 如果是当前页面，且是向前切换
        if event.GetPage() == self and event.GetDirection():
            api_key = self.get_api_key()
            if not api_key:
                wx.MessageBox(T('Please get API Key first'), T('Hint'), wx.OK | wx.ICON_INFORMATION)
                event.Veto()
                return
        event.Skip()
        
class ModelPage(wx.adv.WizardPage):
    def __init__(self, parent, provider_config):
        super().__init__(parent)
        self.provider_config = provider_config
        self.prev_page = None  # 添加一个变量来记录上一页
        self.init_ui()
        self.SetSize(800, 600)

    def init_ui(self):
        vbox = wx.BoxSizer(wx.VERTICAL)

        # 标题
        title = wx.StaticText(self, label=T('Select Model'))
        title.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        vbox.Add(title, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Model 选择
        model_box = wx.StaticBox(self, label=T('Available Models'))
        model_sizer = wx.StaticBoxSizer(model_box, wx.VERTICAL)

        self.model_choice = wx.Choice(self)
        model_sizer.Add(self.model_choice, 0, wx.EXPAND | wx.ALL, 5)
        vbox.Add(model_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # Max Tokens 配置
        max_tokens_box = wx.StaticBox(self, label=T('Max Tokens'))
        max_tokens_sizer = wx.StaticBoxSizer(max_tokens_box, wx.VERTICAL)

        self.max_tokens_text = wx.TextCtrl(self, value="8192")
        max_tokens_sizer.Add(self.max_tokens_text, 0, wx.EXPAND | wx.ALL, 5)
        vbox.Add(max_tokens_sizer, 0, wx.EXPAND | wx.ALL, 5)

        # 提示信息
        self.hint = wx.StaticText(self, label=T('Please select model to use and configure parameters'))
        self.hint.SetForegroundColour(wx.Colour(100, 100, 100))
        vbox.Add(self.hint, 0, wx.ALL, 10)

        self.SetSizer(vbox)

    def set_models(self, models: List[str], selected_model: str = None, is_trust_token: bool = False):
        if is_trust_token:
            self.model_choice.SetItems(["auto"])
            self.model_choice.SetSelection(0)
            self.model_choice.Disable()  # 禁用选择
            self.hint.SetLabel(T('Trustoken uses automatic model selection'))
        else:
            self.model_choice.SetItems(models)
            self.model_choice.Enable()  # 启用选择
            if selected_model and selected_model in models:
                self.model_choice.SetStringSelection(selected_model)
            elif models:  # 如果有模型列表但没有指定选择的模型，默认选择第一个
                self.model_choice.SetSelection(0)
            self.hint.SetLabel(T('Please select model to use and configure parameters'))

    def get_selected_model(self):
        return self.model_choice.GetStringSelection()

    def get_max_tokens(self):
        try:
            return int(self.max_tokens_text.GetValue())
        except ValueError:
            return 8192

    def GetNext(self):
        return None

    def GetPrev(self):
        return self.prev_page

class ProviderConfigWizard(wx.adv.Wizard):
    def __init__(self, llm_config, parent):
        super().__init__(parent, title=T('LLM Provider Configuration Wizard'))
        self.provider_config = llm_config
        self.init_ui()
        self.SetPageSize((600, 400))
        self.Centre()
        self.log = logger.bind(src="wizard")

    def init_ui(self):
        # 创建向导页面
        self.initial_page = InitialProviderPage(self, self.provider_config)
        self.trust_token_page = TrustTokenPage(self, self.provider_config)
        self.provider_page = ProviderPage(self, self.provider_config)
        self.model_page = ModelPage(self, self.provider_config)

        # 绑定事件
        self.Bind(wx.adv.EVT_WIZARD_PAGE_CHANGED, self.on_page_changed)
        self.Bind(wx.adv.EVT_WIZARD_FINISHED, self.on_finished)

        wx.CallAfter(self._set_button_labels)

    def _set_button_labels(self):
        btn_next = self.FindWindowById(wx.ID_FORWARD)
        btn_back = self.FindWindowById(wx.ID_BACKWARD)
        btn_cancel = self.FindWindowById(wx.ID_CANCEL)

        if btn_next: btn_next.SetLabel(T('Next'))
        if btn_back: btn_back.SetLabel(T('Back'))
        if btn_cancel: btn_cancel.SetLabel(T('Cancel'))

    def on_page_changed(self, event):
        btn_next = self.FindWindowById(wx.ID_FORWARD)
        if btn_next: btn_next.SetLabel(T('Next'))

        if event.GetPage() == self.model_page:
            if btn_next: btn_next.SetLabel(T('Finish'))
            # 从第一步进入第二步时，验证 API Key 并获取模型列表
            provider = self.provider_page.get_provider()
            api_key = self.provider_page.get_api_key()
            if not provider or not api_key:
                self.model_page.prev_page = self.trust_token_page
                provider = "Trustoken"
                api_key = self.trust_token_page.get_api_key()
                # 对于 TrustToken，直接设置模型为 auto
                self.model_page.set_models([], None, is_trust_token=True)
            else:
                self.model_page.prev_page = self.provider_page
                models = self.get_models(provider, api_key)
                if not models:
                    event.Veto()
                    return

                # 设置模型列表
                selected_model = None
                if provider in self.provider_config.config:
                    selected_model = self.provider_config.config[provider].get("selected_model")
                self.model_page.set_models(models, selected_model)
        elif event.GetPage() == self.trust_token_page:
            self.trust_token_page.prev_page = self.initial_page
        elif event.GetPage() == self.provider_page:
            self.provider_page.prev_page = self.initial_page
        elif event.GetPage() == self.initial_page:
            self.initial_page.prev_page = None

    def on_finished(self, event):
        # 根据 model_page 的 prev_page 来判断是从哪个路径来的
        if self.model_page.prev_page == self.trust_token_page:
            provider = "Trustoken"
            api_key = self.trust_token_page.get_api_key()
        else:
            provider = self.provider_page.get_provider()
            api_key = self.provider_page.get_api_key()

        selected_model = self.model_page.get_selected_model()
        max_tokens = self.model_page.get_max_tokens()
        provider_info = self.provider_config.providers[provider]

        config = self.provider_config.config
        config[provider] = {
            "api_key": api_key,
            "models": self.model_page.model_choice.GetItems(),
            "model": selected_model,
            "max_tokens": max_tokens,
            "type": provider_info["type"],
            "enable": True
        }

        self.provider_config.save_config(config)
        wx.MessageBox(T('Configuration saved'), T('Success'), wx.OK | wx.ICON_INFORMATION)

    def get_models(self, provider: str, api_key: str) -> List[str]:
        provider_info = self.provider_config.providers[provider]
        headers = {
            "Content-Type": "application/json"
        }

        if provider == "Claude":
            headers["x-api-key"] = api_key
            headers["anthropic-version"] = "2023-06-01"
        else:
            headers["Authorization"] = f"Bearer {api_key}"

        try:
            response = requests.get(
                f"{provider_info['api_base']}{provider_info['models_endpoint']}",
                headers=headers
            )
            self.log.info(f"获取模型列表: {response.text}")
            if response.status_code == 200:
                data = response.json()
                self.log.info(f"获取模型列表成功: {data}")
                if provider in ["OpenAI", "DeepSeek", "xAI", "Claude"]:
                    return [model["id"] for model in data["data"]]
                elif provider == "Gemini":
                    return [model["name"] for model in data["models"]]
                # 其他 provider 的模型解析逻辑
                return []
        except Exception as e:
            wx.MessageBox(f"{T('Model list acquisition failed')}: {str(e)}", T('Error'), wx.OK | wx.ICON_ERROR)
            return []

def show_provider_config(llm_config, parent=None):
    wizard = ProviderConfigWizard(llm_config, parent)
    wizard.RunWizard(wizard.initial_page)
    wizard.Destroy()
    return wx.ID_OK


#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
from functools import wraps

from term_image.image import from_file, from_url

from . import utils
from .plugin import event_bus
from .. import T
from ..exec import BaseRuntime

def restore_output(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        old_stdout, old_stderr = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__

        try:
            return func(self, *args, **kwargs)
        finally:
            sys.stdout, sys.stderr = old_stdout, old_stderr
    return wrapper

class Runtime(BaseRuntime):
    def __init__(self, task):
        super().__init__(task.envs)
        self.gui = task.gui
        self.task = task
        self.console = task.console
        self._auto_install = task.settings.get('auto_install')
        self._auto_getenv = task.settings.get('auto_getenv')

    @restore_output
    def install_packages(self, *packages):
        self.console.print(f"\n⚠️ LLM {T('Request to install third-party packages')}: {packages}")
        ok = utils.confirm(self.console, f"💬 {T('If you agree, please enter')} 'y'> ", auto=self._auto_install)
        if ok:
            ret = self.ensure_packages(*packages)
            self.console.print("\n✅" if ret else "\n❌")
            return ret
        return False
    
    @restore_output
    def get_env(self, name, default=None, *, desc=None):
        self.console.print(f"\n⚠️ LLM {T('Request to obtain environment variable {}, purpose', name)}: {desc}")
        try:
            value = self.envs[name][0]
            self.console.print(f"✅ {T('Environment variable {} exists, returned for code use', name)}")
        except KeyError:
            if self._auto_getenv:
                self.console.print(f"✅ {T('Auto confirm')}")
                value = None
            else:
                value = self.console.input(f"💬 {T('Environment variable {} not found, please enter', name)}: ")
                value = value.strip()
            if value:
                self.set_env(name, value, desc)
        return value or default
    
    @restore_output
    def display(self, path=None, url=None):
        image = {'path': path, 'url': url}
        event_bus.broadcast('display', image)
        if not self.gui:
            image = from_file(path) if path else from_url(url)
            image.draw()

    @restore_output
    def input(self, prompt=''):
        return self.console.input(prompt)    
    
    def get_block_by_name(self, block_name):
        return self.task.code_blocks.get_block_by_name(block_name)


from .trustoken import TrustTokenAuthDialog
from .config import ConfigDialog
from .apimarket import ApiMarketDialog
from .providers import show_provider_config
from .about import AboutDialog
from .statusbar import CStatusBar

__all__ = ['TrustTokenAuthDialog', 'ConfigDialog', 'ApiMarketDialog', 'show_provider_config', 'AboutDialog', 'CStatusBar']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import json
from pathlib import Path
from collections import deque, namedtuple

from loguru import logger

from .. import T
from .task import Task
from .plugin import PluginManager
from .prompt import get_system_prompt
from .diagnose import Diagnose
from .llm import ClientManager
from .config import PLUGINS_DIR, TIPS_DIR, get_mcp, get_tt_api_key, get_tt_aio_api
from .tips import TipsManager

class TaskManager:
    MAX_TASKS = 16

    def __init__(self, settings, console, gui=False):
        self.settings = settings
        self.console = console
        self.tasks = deque(maxlen=self.MAX_TASKS)
        self.envs = {}
        self.gui = gui
        self.log = logger.bind(src='taskmgr')
        self.api_prompt = None
        self.config_files = settings._loaded_files
        self.plugin_manager = PluginManager(PLUGINS_DIR)
        self.plugin_manager.load_plugins()
        if settings.workdir:
            workdir = Path.cwd() / settings.workdir
            workdir.mkdir(parents=True, exist_ok=True)
            os.chdir(workdir)
            self._cwd = workdir
        else:
            self._cwd = Path.cwd()
        self.mcp = get_mcp(settings.get('_config_dir'))
        self._init_environ()
        self.tt_api_key = get_tt_api_key(settings)
        self._init_api()
        self.diagnose = Diagnose.create(settings)
        self.client_manager = ClientManager(settings)
        self.tips_manager = TipsManager(TIPS_DIR)
        self.tips_manager.load_tips()
        self.tips_manager.use(settings.get('role', 'aipy'))
        self.task = None

    @property
    def workdir(self):
        return str(self._cwd)

    def get_tasks(self):
        return list(self.tasks)

    def list_llms(self):
        return self.client_manager.to_records()
    
    def list_envs(self):
        EnvRecord = namedtuple('EnvRecord', ['Name', 'Description', 'Value'])
        rows = []
        for name, (value, desc) in self.envs.items():    
            rows.append(EnvRecord(name, desc, value[:32]))
        return rows
    
    def list_tasks(self):
        rows = []
        for task in self.tasks:
            rows.append(task.to_record())
        return rows
    
    def get_task_by_id(self, task_id):
        for task in self.tasks:
            if task.task_id == task_id:
                return task
        return None

    def get_update(self, force=False):
        return self.diagnose.check_update(force)

    def use(self, llm=None, role=None, task=None):
        if llm:
            ret = self.client_manager.use(llm)
            self.console.print(f"LLM: {'[green]Ok[/green]' if ret else '[red]Error[/red]'}")
        if role:
            ret = self.tips_manager.use(role)
            self.console.print(f"Role: {'[green]Ok[/green]' if ret else '[red]Error[/red]'}")
        if task:
            task = self.get_task_by_id(task)
            self.console.print(f"Task: {'[green]Ok[/green]' if task else '[red]Error[/red]'}")
            self.task = task

    def _init_environ(self):
        envs = self.settings.get('environ', {})
        for name, value in envs.items():
            os.environ[name] = value

    def _init_api(self):
        api = self.settings.get('api', {})

        # update tt aio api, for map and search
        if self.tt_api_key:
            tt_aio_api = get_tt_aio_api(self.tt_api_key)
            api.update(tt_aio_api)

        lines = []
        for api_name, api_conf in api.items():
            lines.append(f"## {api_name} API")
            desc = api_conf.get('desc')
            if desc:
                lines.append(f"### API {T('Description')}\n{desc}")

            envs = api_conf.get('env')
            if not envs:
                continue

            lines.append(f"### {T('Environment variable name and meaning')}")
            for name, (value, desc) in envs.items():
                value = value.strip()
                if not value:
                    continue
                lines.append(f"- {name}: {desc}")
                self.envs[name] = (value, desc)

        self.api_prompt = "\n".join(lines)

    def new_task(self):
        if self.task:
            task = self.task
            self.task = None
            self.log.info('Reload task', task_id=task.task_id)
            return task

        with_mcp = self.settings.get('mcp', {}).get('enable', True)

        mcp_tools = ""
        if self.mcp and with_mcp:
            mcp_tools = self.mcp.get_tools_prompt()
        system_prompt = get_system_prompt(self.tips_manager.current_tips, self.api_prompt, self.settings.get('system_prompt'), mcp_tools=mcp_tools)

        task = Task(self)
        task.client = self.client_manager.Client()
        task.diagnose = self.diagnose
        task.system_prompt = system_prompt
        task.mcp = self.mcp if with_mcp else None
        self.tasks.append(task)
        self.log.info('New task created', task_id=task.task_id)
        return task


from rich import print

from  ... import T
from .base import BaseCommand
from .utils import print_table

class HelpCommand(BaseCommand):
    name = 'help'
    description = T('Show available commands or detailed help for a specific command')

    def add_arguments(self, parser):
        parser.add_argument('target_command', nargs='?', default=None, help='Command to show detailed help for')

    def execute(self, args):
        manager = self.manager
        if args.target_command:
            if args.target_command in manager.commands:
                parser = manager.commands[args.target_command].parser
                print(f"Help for command '{args.target_command}':")
                print(parser.format_help())
            else:
                print(f"Unknown command: {args.target_command}")
        else:
            rows = []
            for cmd, cmd_instance in sorted(manager.commands.items()):
                rows.append([f"/{cmd}", cmd_instance.description])
            print_table(rows, headers=[T('Command'), T('Description')], title=T('Available commands'))
            print()
            print(T("Or directly enter the question to be processed by AI, for example:\n>> Who are you?"))
            print()



import sys

from ... import T
from .base import BaseCommand
from .utils import print_table

class InfoCommand(BaseCommand):
    name = 'info'
    description = T('System information')

    def execute(self, args):
        tm = self.manager.tm
        settings = tm.settings

        info = [
            (T('Current configuration directory'), str(settings.config_dir)),
            (T('Current working directory'), str(tm.workdir)),
            (T('Current LLM'), repr(tm.client_manager.current)),
            (T('Current role'), '-' if settings.get('system_prompt') else tm.tips_manager.current_tips.name),
            ('Python', sys.executable),
            (T('Python version'), sys.version),
            (T('Python base prefix'), sys.base_prefix),
        ]

        print_table(info, title=T("System information"), headers=[T("Parameter"), T("Value")])



from ... import T
from .base import BaseCommand

class UseCommand(BaseCommand):
    name = 'use'
    description = T('Use a LLM or a role')

    def add_arguments(self, parser):
        tm = self.manager.tm
        names = tm.client_manager.names
        roles = {tips.name: tips.role.short for tips in tm.tips_manager.tips.values()}
        parser.add_argument('--llm', choices=names['enabled'], help=T('LLM name'))
        parser.add_argument('--role', choices=roles.keys(), help=T('Role name'))
        parser.add_argument('name', choices=names['enabled'], nargs='?', help=T('LLM name'))

    def execute(self, args):
        tm = self.manager.tm

        params = {}
        if args.llm:
            params['llm'] = args.llm
        if args.role:
            params['role'] = args.role
        if args.name:
            params['llm'] = args.name
        if params:
            tm.use(**params)
        else:
            self.parser.print_help()


from ... import T
from .base import BaseCommand, Completable
from .utils import print_records

class TaskCommand(BaseCommand):
    name = 'task'
    description = T('Task operations')

    def add_subcommands(self, subparsers):
        subparsers.add_parser('list', help='List tasks')
        parser = subparsers.add_parser('use', help='Load an old task')
        parser.add_argument('--tid', type=str, required=True, help='Task ID')

    def cmd_list(self, args):
        rows = self.manager.tm.list_tasks()
        print_records(rows)

    def get_arg_values(self, arg, subcommand=None):
        if subcommand == 'use' and arg.name == '--tid':
            tasks = self.manager.tm.get_tasks()
            return [Completable(task.task_id, task.instruction) for task in tasks]
        return super().get_arg_values(arg, subcommand)
    
    def cmd_use(self, args):
        self.manager.tm.use(task=args.tid)
        self.log.info(f'Use task: {args.tid}')

    def cmd(self, args):
        self.cmd_list(args)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import Counter, defaultdict, namedtuple

from loguru import logger
from rich.live import Live
from rich.text import Text

from .. import T
from .plugin import event_bus
from ..llm import CLIENTS, ChatMessage

class ChatHistory:
    def __init__(self):
        self.messages = []
        self._total_tokens = Counter()

    def __len__(self):
        return len(self.messages)
    
    def json(self):
        return [msg.__dict__ for msg in self.messages]
    
    def add(self, role, content):
        self.add_message(ChatMessage(role=role, content=content))

    def add_message(self, message: ChatMessage):
        self.messages.append(message)
        self._total_tokens += message.usage

    def get_usage(self):
        return iter(row.usage for row in self.messages if row.role == "assistant")
    
    def get_summary(self):
        summary = {'time': 0, 'input_tokens': 0, 'output_tokens': 0, 'total_tokens': 0}
        summary.update(dict(self._total_tokens))
        summary['rounds'] = sum(1 for row in self.messages if row.role == "assistant")
        return summary

    def get_messages(self):
        return [{"role": msg.role, "content": msg.content} for msg in self.messages]

class LineReceiver(list):
    def __init__(self):
        super().__init__()
        self.buffer = ""

    @property
    def content(self):
        return '\n'.join(self)
    
    def feed(self, data: str):
        self.buffer += data
        new_lines = []

        while '\n' in self.buffer:
            line, self.buffer = self.buffer.split('\n', 1)
            self.append(line)
            new_lines.append(line)

        return new_lines
    
    def empty(self):
        return not self and not self.buffer
    
    def done(self):
        buffer = self.buffer
        if buffer:
            self.append(buffer)
            self.buffer = ""
        return buffer

class LiveManager:
    def __init__(self, name, quiet=False):
        self.live = None
        self.name = name
        self.lr = LineReceiver()
        self.lr_reason = LineReceiver()
        self.title = f"{self.name} {T('Reply')}"
        self.reason_started = False
        self.display_lines = []
        self.max_lines = 10
        self.quiet = quiet

    @property
    def content(self):
        return self.lr.content
    
    @property
    def reason(self):
        return self.lr_reason.content
    
    def __enter__(self):
        if self.quiet: return self
        self.live = Live(auto_refresh=False, vertical_overflow='crop', transient=True)
        self.live.__enter__()
        return self

    def process_chunk(self, content, *, reason=False):
        if not content: return
 
        if not reason and self.lr.empty() and not self.lr_reason.empty():
            line = self.lr_reason.done()
            event_bus.broadcast('response_stream', {'llm': self.name, 'content': f"{line}\n\n----\n\n", 'reason': True})

        lr = self.lr_reason if reason else self.lr
        lines = lr.feed(content)
        if not lines: return

        lines2 = [line for line in lines if not line.startswith('<!-- Block-') and not line.startswith('<!-- Cmd-')]
        if lines2:
            content = '\n'.join(lines2)
            event_bus.broadcast('response_stream', {'llm': self.name, 'content': content, 'reason': reason})

        if self.quiet: return

        if reason and not self.reason_started:
            self.display_lines.append("<think>")
            self.reason_started = True
        elif not reason and self.reason_started:
            self.display_lines.append("</think>")
            self.reason_started = False

        self.display_lines.extend(lines)
        while len(self.display_lines) > self.max_lines:
            self.display_lines.pop(0)
        content = '\n'.join(self.display_lines)
        self.live.update(Text(content, style="dim white"), refresh=True)

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.lr.buffer:
            self.process_chunk('\n')
        if self.live:
            self.live.__exit__(exc_type, exc_val, exc_tb)

class ClientManager(object):
    MAX_TOKENS = 8192

    def __init__(self, settings):
        self.clients = {}
        self.default = None
        self.current = None
        self.log = logger.bind(src='client_manager')
        self.names = self._init_clients(settings)

    def _create_client(self, config):
        proto = config.get("type", "openai")
        client = CLIENTS.get(proto.lower())
        if not client:
            self.log.error('Unsupported LLM provider', proto=proto)
            return None
        return client(config)
    
    def _init_clients(self, settings):
        names = defaultdict(set)
        max_tokens = settings.get('max_tokens', self.MAX_TOKENS)
        for name, config in settings.llm.items():
            if not config.get('enable', True):
                names['disabled'].add(name)
                continue
            
            config['name'] = name
            try:
                client = self._create_client(config)
            except Exception as e:
                self.log.exception('Error creating LLM client', config=config)
                names['error'].add(name)
                continue

            if not client or not client.usable():
                names['disabled'].add(name)
                self.log.error('LLM client not usable', name=name, config=config)
                continue

            names['enabled'].add(name)
            if not client.max_tokens:
                client.max_tokens = max_tokens
            self.clients[name] = client

            if config.get('default', False) and not self.default:
                self.default = client
                names['default'] = name

        if not self.default:
            name = list(self.clients.keys())[0]
            self.default = self.clients[name]
            names['default'] = name

        self.current = self.default
        return names

    def __len__(self):
        return len(self.clients)
    
    def __repr__(self):
        return f"Current: {'default' if self.current == self.default else self.current}, Default: {self.default}"
    
    def __contains__(self, name):
        return name in self.clients
    
    def use(self, name):
        client = self.clients.get(name)
        if client and client.usable():
            self.current = client
            return True
        return False

    def get_client(self, name):
        return self.clients.get(name)
    
    def Client(self):
        return Client(self)
    
    def to_records(self):
        LLMRecord = namedtuple('LLMRecord', ['Name', 'Model', 'Max_Tokens', 'Base_URL'])
        rows = []
        for name, client in self.clients.items():
            rows.append(LLMRecord(name, client.model, client.max_tokens, client.base_url))
        return rows
    
class Client:
    def __init__(self, manager: ClientManager):
        self.manager = manager
        self.current = manager.current
        self.history = ChatHistory()
        self.log = logger.bind(src='client', name=self.current.name)

    @property
    def name(self):
        return self.current.name
    
    def use(self, name):
        client = self.manager.get_client(name)
        if client and client.usable():
            self.current = client
            self.log = logger.bind(src='client', name=self.current.name)
            return True
        return False
    
    def __call__(self, instruction, *, system_prompt=None, quiet=False):
        client = self.current
        stream_processor = LiveManager(client.name, quiet=quiet)
        msg = client(self.history, instruction, system_prompt=system_prompt, stream_processor=stream_processor)
        if msg:
            event_bus.broadcast('response_complete', {'llm': client.name, 'content': msg})
        else:
            self.log.error(f"LLM: {client.name} response is None")
        return msg



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import List, Dict, Any, Optional
from dataclasses import dataclass, field
import tomllib
import os

from loguru import logger

@dataclass
class Tip:
    """提示信息对象"""
    name: str
    short: str
    detail: str

    @classmethod
    def from_dict(cls, name: str, data: Dict[str, str]) -> 'Tip':
        """从字典创建提示信息对象"""
        return cls(
            name=name,
            short=data.get('short', ''),
            detail=data.get('detail', '')
        )
    
    def __str__(self):
        return f"<{self.name}>\n{self.detail.strip()}\n</{self.name}>"

class Tips:
    """提示信息管理器"""
    def __init__(self):
        self.role: Tip
        self.tips: Dict[str, Tip] = {}

    @property
    def name(self):
        return self.role.name
    
    def add_tip(self, tip: Tip):
        self.tips[tip.name] = tip

    def get_tip(self, name: str) -> Optional[Tip]:
        """获取指定名称的提示信息"""
        return self.tips.get(name)

    def __iter__(self):
        return iter(self.tips.items())

    def __len__(self):
        return len(self.tips)

    def __getitem__(self, name: str) -> Tip:
        return self.tips[name]
    
    def __str__(self):
        lines = ['<tips>']
        for name, tip in self.tips.items():
            lines.append(str(tip))
        lines.append('</tips>')
        return '\n'.join(lines)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Tips':
        """从字典创建提示信息管理器"""
        tips = cls()
        tips_data = data.get('tips', {})
        for tip_name, tip_data in tips_data.items():
            tip = Tip.from_dict(tip_name, tip_data)
            if tip_name == 'role':
                tip.name = tip_data.get('name', '')
                tips.role = tip
            else:
                tips.add_tip(tip)
        return tips

    @classmethod
    def load(cls, toml_path: Optional[str] = None) -> 'Tips':
        """从 TOML 文件加载提示信息
        
        Args:
            toml_path: TOML 文件路径，如果为 None 则使用默认路径
            
        Returns:
            Tips: 提示信息管理器
        """
        with open(toml_path, 'rb') as f:
            data = tomllib.load(f)
        
        return cls.from_dict(data)

class TipsManager:
    def __init__(self, tips_dir: str = None):
        self.tips_dir = tips_dir
        self.tips: Dict[str, Tips] = {}
        self.default_tips: Tips
        self.current_tips: Tips
        self.log = logger.bind(src='tips')

    def load_tips(self):
        sys_tips_dir = os.path.join(os.path.dirname(__file__), '..', 'res', 'tips')
        for tips_dir in [sys_tips_dir, self.tips_dir]:
            if not tips_dir or not os.path.exists(tips_dir):
                continue
            for fname in os.listdir(tips_dir):
                if fname.endswith(".toml") and not fname.startswith("_"):
                    tips = Tips.load(os.path.join(tips_dir, fname))
                    self.log.info(f"Loaded tips: {tips.name}/{len(tips)}")
                self.tips[tips.name.lower()] = tips

        self.default_tips = self.tips['aipy']
        self.current_tips = self.default_tips

    def use(self, name: str):
        name = name.lower()
        if name in self.tips:
            self.log.info(f"Using tips: {name}")
            self.current_tips = self.tips[name]
            return True
        return False

if __name__ == '__main__':
    # 创建角色实例
    tips_manager = TipsManager()
    tips_manager.load_tips()
    for name, tips in tips_manager.tips.items():
        role = tips.role
        print(name)
        print(f"Tips: {name}")
        print(f"Role: {role.name}")
        print(f"Short: {role.short}")
        print(f"Detail: {role.detail}")
        print("-" * 100)

        # 打印角色信息
        print(f"角色名称: {role.name}")
        print(f"简短描述: {role.short}")
        print(f"详细描述: {role.detail}")
        
        # 打印所有提示信息
        print("\n提示信息:")
        for name, tip in tips:
            print(f"\n{tip.name}:")
            print(f"简短描述: {tip.short}")
            print(f"详细描述: {tip.detail}")


from importlib import resources

from .i18n import T, set_lang, get_lang

__version__ = '0.2.2'

__respkg__ = f'{__package__}.res'
__respath__ = resources.files(__respkg__)

__all__ = ['T', 'set_lang', 'get_lang', '__version__', '__respkg__', '__respath__']


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import json
import uuid
import time
from pathlib import Path
from datetime import datetime
from collections import namedtuple
from importlib.resources import read_text

import requests
from loguru import logger
from rich.rule import Rule
from rich.panel import Panel
from rich.align import Align
from rich.table import Table
from rich.syntax import Syntax
from rich.console import Console, Group
from rich.markdown import Markdown

from .. import T, __respkg__
from ..exec import Runner
from .runtime import Runtime
from .plugin import event_bus
from .utils import get_safe_filename
from .blocks import CodeBlocks, CodeBlock
from .interface import Stoppable
from . import prompt

CONSOLE_WHITE_HTML = read_text(__respkg__, "console_white.html")
CONSOLE_CODE_HTML = read_text(__respkg__, "console_code.html")

class Task(Stoppable):
    MAX_ROUNDS = 16

    def __init__(self, manager):
        super().__init__()
        self.manager = manager
        self.task_id = uuid.uuid4().hex
        self.log = logger.bind(src='task', id=self.task_id)
        self.settings = manager.settings
        self.envs = manager.envs
        self.gui = manager.gui
        self.console = Console(file=manager.console.file, record=True)
        self.max_rounds = self.settings.get('max_rounds', self.MAX_ROUNDS)

        self.client = None
        self.runner = None
        self.instruction = None
        self.system_prompt = None
        self.diagnose = None
        self.start_time = None
        self.done_time = None
        self.code_blocks = CodeBlocks(self.console)
        self.runtime = Runtime(self)
        self.runner = Runner(self.runtime)
        
    def to_record(self):
        TaskRecord = namedtuple('TaskRecord', ['task_id', 'start_time', 'done_time', 'instruction'])
        start_time = datetime.fromtimestamp(self.start_time).strftime('%H:%M:%S') if self.start_time else '-'
        done_time = datetime.fromtimestamp(self.done_time).strftime('%H:%M:%S') if self.done_time else '-'
        return TaskRecord(
            task_id=self.task_id,
            start_time=start_time,
            done_time=done_time,
            instruction=self.instruction[:32] if self.instruction else '-'
        )
    
    def use(self, name):
        ret = self.client.use(name)
        #self.console.print('[green]Ok[/green]' if ret else '[red]Error[/red]')
        return ret

    def save(self, path):
       if self.console.record:
           self.console.save_html(path, clear=False, code_format=CONSOLE_WHITE_HTML)

    def save_html(self, path, task):
        if 'chats' in task and isinstance(task['chats'], list) and len(task['chats']) > 0:
            if task['chats'][0]['role'] == 'system':
                task['chats'].pop(0)

        task_json = json.dumps(task, ensure_ascii=False, default=str)
        html_content = CONSOLE_CODE_HTML.replace('{{code}}', task_json)
        try:
            with open(path, 'w', encoding='utf-8') as f:
                f.write(html_content)
        except Exception as e:
            self.console.print_exception()
        
    def _auto_save(self):
        instruction = self.instruction
        task = {'instruction': instruction}
        task['chats'] = self.client.history.json()
        #task['envs'] = self.runtime.envs
        task['runner'] = self.runner.history
        task['blocks'] = self.code_blocks.to_list()

        filename = f"{self.task_id}.json"
        try:
            json.dump(task, open(filename, 'w', encoding='utf-8'), ensure_ascii=False, indent=4, default=str)
        except Exception as e:
            self.log.exception('Error saving task')

        filename = f"{self.task_id}.html"
        #self.save_html(filename, task)
        self.save(filename)
        self.log.info('Task auto saved')

    def done(self):
        curname = f"{self.task_id}.json"
        jsonname = get_safe_filename(self.instruction, extension='.json')
        if jsonname and os.path.exists(curname):
            try:
                os.rename(curname, jsonname)
            except Exception as e:
                self.log.exception('Error renaming task json file')

        curname = f"{self.task_id}.html"
        htmlname = get_safe_filename(self.instruction, extension='.html')
        if htmlname and os.path.exists(curname):
            try:
                os.rename(curname, htmlname)
            except Exception as e:
                self.log.exception('Error renaming task html file')

        self.diagnose.report_code_error(self.runner.history)
        self.done_time = time.time()
        self.log.info('Task done', jsonname=jsonname, htmlname=htmlname)
        filename = str(Path(htmlname).resolve())
        self.console.print(f"[green]{T('Result file saved')}: \"{filename}\"")
        if self.settings.get('share_result'):
            self.sync_to_cloud()
        
    def process_reply(self, markdown):
        #self.console.print(f"{T('Start parsing message')}...", style='dim white')
        parse_mcp = self.mcp is not None
        ret = self.code_blocks.parse(markdown, parse_mcp=parse_mcp)
        if not ret:
            return None
        
        json_str = json.dumps(ret, ensure_ascii=False, indent=2, default=str)
        self.box(f"✅ {T('Message parse result')}", json_str, lang="json")

        if 'call_tool' in ret:
            # 有可能MCP调用会按照代码格式返回，这种情况下存在exec_blocks和call_tool两个键,也可能会有erros字段
            # 优先处理call_tool
            return self.process_mcp_reply(ret['call_tool'])

        errors = ret.get('errors')
        if errors:
            event_bus('result', errors)
            self.console.print(f"{T('Start sending feedback')}...", style='dim white')
            feed_back = f"# 消息解析错误\n{json_str}"
            ret = self.chat(feed_back)
        elif 'exec_blocks' in ret:
            ret = self.process_code_reply(ret['exec_blocks'])
        else:
            ret = None
        return ret

    def print_code_result(self, block, result, title=None):
        line_numbers = True if 'traceback' in result else False
        syntax_code = Syntax(block.code, block.lang, line_numbers=line_numbers, word_wrap=True)
        json_result = json.dumps(result, ensure_ascii=False, indent=2, default=str)
        syntax_result = Syntax(json_result, 'json', line_numbers=False, word_wrap=True)
        group = Group(syntax_code, Rule(), syntax_result)
        panel = Panel(group, title=title or block.name)
        self.console.print(panel)

    def process_code_reply(self, exec_blocks):
        results = []
        for block in exec_blocks:
            event_bus('exec', block)
            self.console.print(f"⚡ {T('Start executing code block')}: {block.name}", style='dim white')
            result = self.runner(block)
            self.print_code_result(block, result)
            result['block_name'] = block.name
            results.append(result)
            event_bus('result', result)

        msg = prompt.get_results_prompt(results)
        self.console.print(f"{T('Start sending feedback')}...", style='dim white')
        feed_back = json.dumps(msg, ensure_ascii=False, default=str)
        return self.chat(feed_back)

    def process_mcp_reply(self, json_content):
        """处理 MCP 工具调用的回复"""
        block = {'content': json_content, 'language': 'json'}
        event_bus('tool_call', block)
        self.console.print(f"⚡ {T('Start calling MCP tool')} ...", style='dim white')

        call_tool = json.loads(json_content)
        result = self.mcp.call_tool(call_tool['name'], call_tool.get('arguments', {}))
        event_bus('result', result)
        code_block = CodeBlock(
            code=json_content,
            lang='json',
            name=call_tool.get('name', 'MCP Tool Call'),
            version=1,
        )
        self.print_code_result(code_block, result, title=T("MCP tool call result"))

        self.console.print(f"{T('Start sending feedback')}...", style='dim white')
        msg = prompt.get_mcp_result_prompt(result)
        feed_back = json.dumps(msg, ensure_ascii=False, default=str)
        feedback_response = self.chat(feed_back)
        return feedback_response

    def box(self, title, content, align=None, lang=None):
        if lang:
            content = Syntax(content, lang, line_numbers=True, word_wrap=True)
        else:
            content = Markdown(content)

        if align:
            content = Align(content, align=align)
        
        self.console.print(Panel(content, title=title))

    def print_summary(self, detail=False):
        history = self.client.history
        if detail:
            table = Table(title=T("Task Summary"), show_lines=True)

            table.add_column(T("Round"), justify="center", style="bold cyan", no_wrap=True)
            table.add_column(T("Time(s)"), justify="right")
            table.add_column(T("In Tokens"), justify="right")
            table.add_column(T("Out Tokens"), justify="right")
            table.add_column(T("Total Tokens"), justify="right", style="bold magenta")

            round = 1
            for row in history.get_usage():
                table.add_row(
                    str(round),
                    str(row["time"]),
                    str(row["input_tokens"]),
                    str(row["output_tokens"]),
                    str(row["total_tokens"]),
                )
                round += 1
            self._console.print("\n")
            self._console.print(table)

        summary = history.get_summary()
        summary['elapsed_time'] = time.time() - self.start_time
        summarys = "| {rounds} | {time:.3f}s/{elapsed_time:.3f}s | Tokens: {input_tokens}/{output_tokens}/{total_tokens}".format(**summary)
        event_bus.broadcast('summary', summarys)
        self.console.print(f"\n⏹ [cyan]{T('End processing instruction')} {summarys}")


    def chat(self, instruction, *, system_prompt=None):
        quiet = self.settings.gui and not self.settings.debug
        msg = self.client(instruction, system_prompt=system_prompt, quiet=quiet)
        if msg.role == 'error':
            self.console.print(f"[red]{msg.content}[/red]")
            return None
        if msg.reason:
            content = f"{msg.reason}\n\n-----\n\n{msg.content}"
        else:
            content = msg.content
        self.box(f"[yellow]{T('Reply')} ({self.client.name})", content)
        return msg.content

    def run(self, instruction):
        """
        执行自动处理循环，直到 LLM 不再返回代码消息
        """
        self.box(f"[yellow]{T('Start processing instruction')}", instruction, align="center")
        if not self.start_time:
            self.start_time = time.time()
            self.instruction = instruction
            msg = prompt.get_task_prompt(instruction, gui=self.gui)
            event_bus('task_start', prompt)
            system_prompt = self.system_prompt
        else:
            system_prompt = None
            msg = prompt.get_chat_prompt(instruction, self.instruction)

        rounds = 1
        max_rounds = self.max_rounds
        json_prompt = json.dumps(msg, ensure_ascii=False, default=str)
        response = self.chat(json_prompt, system_prompt=system_prompt)
        while response and rounds <= max_rounds:
            response = self.process_reply(response)
            rounds += 1
            if self.is_stopped():
                self.log.info('Task stopped')
                break

        self.print_summary()
        self._auto_save()
        self.console.bell()
        self.log.info('Loop done', rounds=rounds)

    def sync_to_cloud(self, verbose=True):
        """ Sync result
        """
        url = T("https://store.aipy.app/api/work")

        trustoken_apikey = self.settings.get('llm', {}).get('Trustoken', {}).get('api_key')
        if not trustoken_apikey:
            trustoken_apikey = self.settings.get('llm', {}).get('trustoken', {}).get('api_key')
        if not trustoken_apikey:
            return False
        self.console.print(f"[yellow]{T('Uploading result, please wait...')}")
        try:
            # Serialize twice to remove the non-compliant JSON type.
            # First, use the json.dumps() `default` to convert the non-compliant JSON type to str.
            # However, NaN/Infinity will remain.
            # Second, use the json.loads() 'parse_constant' to convert NaN/Infinity to str.
            data = json.loads(
                json.dumps({
                    'apikey': trustoken_apikey,
                    'author': os.getlogin(),
                    'instruction': self.instruction,
                    'llm': self.client.history.json(),
                    'runner': self.runner.history,
                }, ensure_ascii=False, default=str),
                parse_constant=str)
            response = requests.post(url, json=data, verify=True,  timeout=30)
        except Exception as e:
            print(e)
            return False

        status_code = response.status_code
        if status_code in (200, 201):
            if verbose:
                data = response.json()
                url = data.get('url', '')
                if url:
                    self.console.print(f"[green]{T('Article uploaded successfully, {}', url)}[/green]")
            return True

        if verbose:
            self.console.print(f"[red]{T('Upload failed (status code: {})', status_code)}:", response.text)
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import threading
from abc import ABC, abstractmethod

class Runtime(ABC):
    @abstractmethod
    def install_packages(self, packages):
        pass

    @abstractmethod
    def getenv(self, name, desc=None):
        pass

class ConsoleInterface(ABC):
    @abstractmethod
    def print(self, *args, sep=' ', end='\n', file=None, flush=False):
        pass

    @abstractmethod
    def input(self, prompt=''):
        pass

    @abstractmethod
    def status(self, msg):
        pass

class Stoppable():
    def __init__(self):
        self._stop_event = threading.Event()

    def on_stop(self):
        pass

    def stop(self):
        self._stop_event.set()
        self.on_stop()
        
    def is_stopped(self):
        return self._stop_event.is_set()
    
    def wait(self, timeout=None):
        return self._stop_event.wait(timeout)
    
    def reset(self):
        self._stop_event.clear()


from rich import print

from ... import T
from .base import BaseCommand

class MCPCommand(BaseCommand):
    name = 'mcp'
    description = T('MCP operations')

    def process_mcp_ret(self, arg, ret):
        if ret.get("status", "success") == "success":
            mcp_status = T('Enabled') if ret.get("globally_enabled") else T('Disabled')
            print(f"[green]{T('MCP server status: {}').format(mcp_status)}[/green]")
            mcp_servers = ret.get("servers", [])
            if ret.get("globally_enabled", False):
                for server_name, info in mcp_servers.items():
                    server_status = T('Enabled') if info.get("enabled", False) else T('Disabled')
                    print(
                        "[", server_status, "]",
                        server_name, info.get("tools_count"), T("Tools")
                    )
        else:
            print("操作失败", ret.get("message", ''))

    def add_subcommands(self, subparsers):
        subparsers.add_parser('list', help=T('List MCP servers'))
        parser = subparsers.add_parser('enable', help=T('Enable MCP server'))
        parser.add_argument('server', nargs='?', default=None, help=T('MCP server name'))
        parser = subparsers.add_parser('disable', help=T('Disable MCP server'))
        parser.add_argument('server', nargs='?', default=None, help=T('MCP server name'))

    def execute(self, args):
        print(args.raw_args)
        raw_args = args.raw_args
        if not raw_args:
            return
        
        tm = self.manager.tm
        if not tm.mcp:
            self.log.error('MCP not found')
            return
            
        ret = tm.mcp.process_command(raw_args)
        self.process_mcp_ret(raw_args, ret)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import json
import traceback
from io import StringIO
import webbrowser

from loguru import logger

INIT_IMPORTS = """
import os
import re
import sys
import json
import time
import random
import traceback

__result__ = {}
__storage__ = {}

def set_result(**kwargs):
    global __result__
    __result__.update(kwargs)

def set_persistent_state(**kwargs):
    global __storage__
    __storage__.update(kwargs)

def get_persistent_state(key):
    global __storage__
    return __storage__.get(key)
"""

def is_json_serializable(obj):
    try:
        json.dumps(obj, ensure_ascii=False, default=str)
        return True
    except (TypeError, OverflowError):
        return False

def diff_dicts(dict1, dict2):
    diff = {}
    for key, value in dict1.items():
        if key not in dict2:
            diff[key] = value
            continue

        try:
            if value != dict2[key]:
                diff[key] = value
        except Exception:
            pass
    return diff

class Runner():
    def __init__(self, runtime):
        self.runtime = runtime
        self.history = []
        self.log = logger.bind(src='runner')
        self._globals = {'runtime': runtime, '__name__': '__main__', 'input': self.runtime.input}
        exec(INIT_IMPORTS, self._globals)

    def __repr__(self):
        return f"<Runner history={len(self.history)}, env={len(self.env)}>"
    
    @property
    def globals(self):
        return self._globals
    
    def _exec_python_block(self, block, params=None):
        old_stdout, old_stderr = sys.stdout, sys.stderr
        captured_stdout = StringIO()
        captured_stderr = StringIO()
        sys.stdout, sys.stderr = captured_stdout, captured_stderr
        result = {}
        env = self.runtime.envs.copy()
        session = self._globals['__storage__'].copy()
        gs = self._globals.copy()
        gs['__result__'].clear()
        gs['__params__'] = params
        try:
            exec(block.code, gs)
        except (SystemExit, Exception) as e:
            result['errstr'] = str(e)
            result['traceback'] = traceback.format_exc()
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr

        s = captured_stdout.getvalue().strip()
        if s: result['stdout'] = s if is_json_serializable(s) else '<filtered: cannot json-serialize>'
        s = captured_stderr.getvalue().strip()
        if s: result['stderr'] = s if is_json_serializable(s) else '<filtered: cannot json-serialize>'        

        vars = gs.get('__result__')
        if vars:
            #self._globals['__retval__'] = vars
            result['result'] = self.filter_result(vars)

        history = {}
        diff = diff_dicts(env, self.runtime.envs)
        if diff:
            history['env'] = diff
        diff = diff_dicts(gs['__storage__'], session)
        if diff:
            history['session'] = diff

        return result, history

    def __call__(self, block):
        self.log.info(f'Exec: {block}')
        lang = block.get_lang()
        if lang == 'python':
            result, history = self._exec_python_block(block)
        elif lang == 'html':
            result, history = self._exec_html_block(block)
        else:
            result = {'stderr': f'Exec: Ignore unsupported block type: {lang}'}
            history = {}

        history['block_name'] = block.name
        history['result'] = result
        self.history.append(history)
        return result.copy()
        
    def _exec_html_block(self, block):
        abs_path = block.abs_path
        if abs_path:
            webbrowser.open(f'file://{abs_path}')
        result = {'stdout': 'OK'}
        return result, {}

    def filter_result(self, vars):
        if isinstance(vars, dict):
            for key in vars.keys():
                if key in self.runtime.envs:
                    vars[key] = '<masked>'
                else:
                    vars[key] = self.filter_result(vars[key])
        elif isinstance(vars, list):
            vars = [self.filter_result(v) for v in vars]
        else:
            vars = vars if is_json_serializable(vars) else '<filtered>'
        return vars
    